
/* 
 * File:   GSLMatrix.h
 * Author: skidmarks
 *
 * Created on December 15, 2017, 2:54 PM
 */

#ifndef GSLMATRIX_H
#define GSLMATRIX_H

# include <gsl/gsl_matrix.h>

# include "GSLData.h"
# include "GSLVector.h"

using namespace std;
using namespace gsl;

namespace gsl {
   
class GSLMatrix : public GSLData{
private:
public:
   GSLMatrix(size_t row, size_t col);
   GSLMatrix(size_t row, size_t col, int calloc);
   GSLMatrix(_gsl_matrix_view matrix);
   GSLMatrix(_gsl_matrix_const_view matrix);
   GSLMatrix(const GSLMatrix& orig);
   virtual ~GSLMatrix();
   virtual string toString() { return ""; };
   
    
   /******************************************************
    *        Common Matrix and Vector properties         *
    ******************************************************/
   virtual void*    data()                                     { return ((gsl_matrix*)block())->data;}
   virtual double   gslmax()                                   { return gsl_matrix_max((gsl_matrix*)block());};
   virtual double   gslmin()                                   { return gsl_matrix_min((gsl_matrix*)block());}
   
   virtual int      isnull()                                   { return gsl_matrix_isnull((gsl_matrix*)block());};
   virtual int      isneg()                                    { return gsl_matrix_isneg((gsl_matrix*)block());};
   virtual int      isnonneg()                                 { return gsl_matrix_isnonneg((gsl_matrix*)block());};
   virtual int      ispos()                                    { return gsl_matrix_ispos((gsl_matrix*)block());};\

   /******************************************************
    *    Accessing & initializing Matrices and Vectors   *
    ******************************************************/
   virtual double*  ptr(size_t row, size_t col)                { return gsl_matrix_ptr((gsl_matrix*)block(), row, col);};
   virtual const double * const_ptr(size_t row, size_t col)    { return gsl_matrix_const_ptr((gsl_matrix*)block(), row, col);};
   virtual double get(const size_t row, const size_t col)      { return gsl_matrix_get((gsl_matrix*)block(), row, col);};
   virtual void   set_all(double x)                            { gsl_matrix_set_all((gsl_matrix*)block(), x);};
   virtual void   set_zero()                                   { gsl_matrix_set_zero((gsl_matrix*)block());};
   virtual void   set(const double x, const size_t row, const size_t col)
                                    { return gsl_matrix_set((gsl_matrix*)block(), row, col, x);};
      
   /******************************************************
    *            Matrix and Vector Arithmetic            *
    ******************************************************/
   virtual int add(GSLMatrix& b)                         { return gsl_matrix_add((gsl_matrix*)block(), (gsl_matrix*)b.block());};
   virtual int add_constant(const double x)              { return gsl_matrix_add_constant((gsl_matrix*)block(), x);};
   virtual int sub(GSLMatrix& b)                         { return gsl_matrix_sub((gsl_matrix*)block(), (gsl_matrix*)b.block());};
   virtual int scale(const double x)                     { return gsl_matrix_scale((gsl_matrix*)block(), x);};
      
   /******************************************************
    *            Matrix and Vector Operations            *
    ******************************************************/
   virtual int    equal(GSLMatrix& b)                    { return gsl_matrix_equal((gsl_matrix*)block(), (gsl_matrix*)b.block());};;
   virtual void   minmax(double& min, double& max)       { gsl_matrix_minmax((gsl_matrix*)block(), &min, &max); };
   virtual void   max_index(size_t& row, size_t& col)    { gsl_matrix_max_index((gsl_matrix*)block(), &row, &col);};
   virtual void   min_index(size_t& row, size_t& col)    { gsl_matrix_min_index((gsl_matrix*)block(), &row, &col);};
   virtual void   minmax_index(size_t& row1, size_t& col1, size_t& row2, size_t& col2)
               { gsl_matrix_minmax_index((gsl_matrix*)block(), &row1, &col1, &row2, &col2);};
   
   /******************************************************
    *            Copying Matrices and Vectors            *
    ******************************************************/
   virtual int memcpy(GSLMatrix& dest)                   { return gsl_matrix_memcpy((gsl_matrix*)dest.block(), (gsl_matrix*)block());};
   virtual int swap(gsl_matrix * m)                      { return gsl_matrix_swap(m, (gsl_matrix*)block());};
  
   /******************************************************
    *                 Common Views                       *
    ******************************************************/
   virtual GSLVector view_array(double* base, const size_t number, size_t col) { return GSLVector(gsl_vector_view_array(base, number));};
   virtual const GSLVector view_array(const double* base, const size_t number, size_t col) { return GSLVector(gsl_vector_const_view_array(base, number));};
            
   /******************************************************
    *          Accessing & initializing Matrices         *
    ******************************************************/
   virtual void set_identity()                           { gsl_matrix_set_identity((gsl_matrix*)block());};
  
   /******************************************************
    *                    Matrix Views                    *
    ******************************************************/
   virtual GSLMatrix submatrix(size_t row, size_t col, size_t rows, size_t cols);
   virtual GSLVector subrow(size_t row, size_t offset, size_t number);
   virtual GSLVector view_array_with_tda(double * base, size_t rows, size_t cols, size_t tda);
   virtual GSLMatrix view_vector(GSLVector& v, size_t rows, size_t cols);
   virtual GSLMatrix view_vector_with_tda(GSLVector& v, size_t rows, size_t cols, size_t tda);
  
   /******************************************************
    *                 const Matrix Views                 *
    ******************************************************/
   virtual const GSLMatrix const_submatrix(size_t row, size_t col, size_t rows, size_t cols);
   virtual const GSLMatrix const_view_array_with_tda(double * base, size_t rows, size_t cols, size_t tda);
   virtual const GSLMatrix const_view_vector(GSLVector& v, size_t rows, size_t cols);
   virtual const GSLMatrix const_view_vector_with_tda(GSLVector& v, size_t rows, size_t cols, size_t tda);
  
   /******************************************************
    *            Row and Column Matrix Views             *
    ******************************************************/
   virtual GSLVector row(size_t row)                     { return GSLVector(gsl_matrix_row((gsl_matrix*)block(), row));};
   virtual GSLVector col(size_t col)                     { return GSLVector(gsl_matrix_column((gsl_matrix*)block(), col));};
   virtual GSLVector subcol(size_t col, size_t offset, size_t number);
   virtual GSLVector diagonal()                          { return GSLVector(gsl_matrix_diagonal((gsl_matrix*)block()));};
   virtual GSLVector subdiagonal(size_t diagonal)        { return GSLVector(gsl_matrix_subdiagonal((gsl_matrix*)block(), diagonal));};
   virtual GSLVector superdiagonal(size_t diagonal)      { return GSLVector(gsl_matrix_superdiagonal((gsl_matrix*)block(), diagonal));};
  
   /******************************************************
    *        const Row and Column Matrix Views           *
    ******************************************************/
   virtual GSLVector const_row(size_t row)               { return GSLVector(gsl_matrix_const_row((gsl_matrix*)block(), row));};
   virtual GSLVector const_col(size_t col)               { return GSLVector(gsl_matrix_const_column((gsl_matrix*)block(), col));};
   virtual GSLVector const_subrow(size_t row, size_t offset, size_t number) 
                                        { return GSLVector(gsl_matrix_const_subrow((gsl_matrix*)block(), row, offset, number));};
   virtual GSLVector const_subcol(size_t col, size_t offset, size_t number) 
                                        { return GSLVector(gsl_matrix_const_subcolumn((gsl_matrix*)block(), col, offset, number));};
   virtual GSLVector const_diagonal()                    { return GSLVector(gsl_matrix_const_diagonal((gsl_matrix*)block()));};
   virtual GSLVector const_subdiagonal(size_t diagonal)  { return GSLVector(gsl_matrix_const_subdiagonal((gsl_matrix*)block(), diagonal));};
   virtual GSLVector const_superdiagonal(size_t diagonal){ return GSLVector(gsl_matrix_const_superdiagonal((gsl_matrix*)block(), diagonal));};
  
   
   /******************************************************
    *          Matrix Copying Rows and Columns           *
    ******************************************************/
   virtual int get_row(GSLVector& v, const size_t row)   { return gsl_matrix_get_row((gsl_vector*)(v.block()), (gsl_matrix*)block(), row);};
   virtual int get_col(GSLVector& v, const size_t col)   { return gsl_matrix_get_col((gsl_vector*)(v.block()), (gsl_matrix*)block(), col);};
   virtual int set_row(const size_t row, GSLVector& v)   { return gsl_matrix_get_row((gsl_vector*)(v.block()), (gsl_matrix*)block(), row);};
   virtual int set_col(const size_t col, GSLVector& v)   { return gsl_matrix_get_row((gsl_vector*)(v.block()), (gsl_matrix*)block(), col);};
  
   
   /******************************************************
    *        Matrix Exchanging Rows and Columns          *
    ******************************************************/
   virtual int swap_rows(size_t row1, size_t row2)       { return gsl_matrix_swap_rows(((gsl_matrix*)block()), row1, row2);};
   virtual int swap_columns(size_t col1, size_t col2)    { return gsl_matrix_swap_columns((gsl_matrix*)block(), col1, col2 );};
   virtual int swap_rowcol(size_t row, size_t col)       { return gsl_matrix_swap_rowcol((gsl_matrix*)block(), row, col );};
   virtual int transpose_memcpy(gsl_matrix * dest)       { return gsl_matrix_transpose_memcpy(dest, (gsl_matrix*)block());};
   virtual int transpose()                               { return gsl_matrix_transpose((gsl_matrix*)block());};
   
   /******************************************************
    *                  Matrix Operations                 *
    ******************************************************/
   virtual int mul_elements(GSLMatrix& b)                { return gsl_matrix_mul_elements((gsl_matrix*)block(), (gsl_matrix*)b.block());};
   virtual int div_elements(GSLMatrix& b)                { return gsl_matrix_div_elements((gsl_matrix*)block(), (gsl_matrix*)b.block());};
        
private:
//   GSLMatrix(const GSLMatrix& orig);

};

} // namespace gls
#endif /* GSLMATRIX_H */

